[cylc]
    UTC mode = True
    [[events]]
        inactivity = PT10S
        abort on inactivity = True
        timeout handler = cylc stop %(suite)s
    [[reference test]]
        live mode suite timeout=PT1M

[scheduling]
    initial cycle point = 20100101T0000Z
    final cycle point = +P1D
    [[dependencies]]

        [[[T00]]]
            graph = foo[-P1D] => foo => bar

[runtime]
    [[root]]
    [[foo, bar]]
        script = sleep 30
