title = "Test validation of simple multiple inheritance"
 
description = """Bug identified at 5.1.1-314-g4960684."""

[cylc]
[[reference test]]
required run mode = live
live mode suite timeout = 1.0 # minutes
[scheduling]
[[dependencies]]
graph = """foo"""
[runtime]
[[FOO]]
[[BAR]]
[[foo]]
inherit = FOO, BAR
