[cylc]
   [[reference test]]
       required run mode = live
       live mode suite timeout = 2.0 # minutes
[scheduling]
    initial cycle time = 2010010100
    final cycle time = 2010010200
    [[special tasks]]
        cold-start = cold
    [[dependencies]]
        [[[0, 6]]]
            graph = "cold | c[T-6] => a => b => c"
        [[[18]]]
            graph = "c"
[runtime]
    [[a,c,cold]]
        script = "true"
    [[b]]
        script = "cylc reload $CYLC_SUITE_REG_NAME -f; sleep 5"
