title = "family triggering test suite"

description = """
For manual testing of new-style family triggers. Play around
with script true/false to get different outcomes."""

[scheduling]
    initial cycle point = 20120808T00
    final cycle point = 20120810T00
    [[dependencies]]
        [[[R1]]]
            graph = """
        foo => ENS
        ENS:start-any => start_any
        ENS:start-all => start_all
        ENS:succeed-any => succeed_any
        ENS:succeed-all => succeed_all
        ENS:fail-any => fail_any
        ENS:fail-all => fail_all
        ENS:finish-any => finish_any
        ENS:finish-all => finish_all
        ENS:finish-all & ENS:succeed-any => all_fin_some_succ
                    """
        [[[T00]]]
            graph = "C_ENS[-P1D]:succeed-all => C_ENS"
[runtime]
    [[ENS]]
    [[m1]]
        inherit = ENS
        script = "sleep 10; true"
    [[m2]]
        inherit = ENS
        init-script = "sleep 5" # delay submission
        script = "sleep 15; false"
    [[m3]]
        inherit = ENS
        init-script = "sleep 10" # delay submission
        script = "sleep 20; true"

    [[C_ENS]]
    [[c_m1]]
        inherit = C_ENS
        script = "sleep 10; true"
    [[c_m2]]
        inherit = C_ENS
        init-script = "sleep 5" # delay submission
        script = "sleep 15; true"
    [[c_m3]]
        inherit = C_ENS
        init-script = "sleep 10" # delay submission
        script = "sleep 20; true"
