#!jinja2
[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20141106T09
    final cycle point = 20141106T09
    [[special tasks]]
        clock-triggered = foo(PT5M)
    [[dependencies]]
        [[[PT1H]]]
            graph = """
                show => foo
                foo[-PT1H] => foo
                foo:start => baz & bar
            """
[runtime]
    [[foo]]
        script = sleep 10
    [[bar,baz]]
        script = true
    [[show]]
        script = """
cylc show "$CYLC_SUITE_NAME" foo.20141106T0900Z >{{ TEST_OUTPUT_PATH }}
"""
