#!jinja2
title = Suite-1000-x3
description = A suite of 1000 tasks per cycle
[cylc]
    UTC mode = True # Ignore DST
[scheduling]
    initial cycle point = 20130101T00
    final cycle point = 20130103T00
    runahead limit = PT12H
    [[dependencies]]
        [[[T00]]]
            graph="""
                {% for i0 in range(10) -%}
                {% for i1 in range(10) -%}
                {% for i2 in range(1) -%}
                t{{i0}}{{i1}}{{i2}}9[-P1D] => t{{i0}}{{i1}}{{i2}}0
                t{{i0}}{{i1}}{{i2}}0 => t{{i0}}{{i1}}{{i2}}1 => \
                t{{i0}}{{i1}}{{i2}}2 => t{{i0}}{{i1}}{{i2}}3
                t{{i0}}{{i1}}{{i2}}3 => t{{i0}}{{i1}}{{i2}}4 => \
                t{{i0}}{{i1}}{{i2}}5 => t{{i0}}{{i1}}{{i2}}6
                t{{i0}}{{i1}}{{i2}}6 => t{{i0}}{{i1}}{{i2}}7 => \
                t{{i0}}{{i1}}{{i2}}8 => t{{i0}}{{i1}}{{i2}}9
                {% endfor -%}
                {% endfor -%}
                {% endfor -%}
            """
[runtime]
    [[root]]
        script = sleep 1
        [[[job]]]
            batch system = at
        [[[events]]]
            succeeded handler = true
            failed handler = true
            retry handler = true
            submission failed handler = true
            submission timeout handler = true
            execution timeout handler = true
            execution timeout = PT6M
            submission timeout = PT1M
{% for i0 in range(10) -%}
{% for i1 in range(10) -%}
{% for i2 in range(1) -%}
{% for i3 in range(10) -%}
    [[t{{i0}}{{i1}}{{i2}}{{i3}}]]
{% endfor -%}
{% endfor -%}
{% endfor -%}
{% endfor -%}
