
title = "test insertion of a task with a bad cycle point."

[cylc]
    cycle point time zone = +01
    [[reference test]]
        required run mode = live
        live mode suite timeout = PT10S

[scheduling]
    initial cycle time = 20140101T00
    final cycle time   = 20140101T00 # 3 cycles
    [[special tasks]]
        exclude at start-up = foo
    [[dependencies]]
        [[[R1]]]
            graph = """
                prep
                foo => bar
            """

[runtime]
    [[root]]
        script = "sleep 1" # quick
    [[prep]]
        # Insert the task with a bad cycle point
        script = cylc insert $CYLC_SUITE_NAME foo 20140101T00 soon
