[cylc]
   [[reference test]]
       required run mode = live
       live mode suite timeout = 1.0 # minutes
[scheduling]
    [[dependencies]]
        graph = "a => b => c"
[runtime]
    [[a,b,c]]
        script = true
