title = "One task takes 20 sec to succeed, another to fail."
[cylc]
    [[reference test]]
        live mode suite timeout = PT10M
[scheduling]
    [[dependencies]]
        graph = """
             good & bad
          bad:fail => !bad
                """
[runtime]
    [[good]]
        script = "sleep 20"
    [[bad]]
        script = "sleep 20; /bin/false"
