title = "climate model run"
description = """
    A climate model run with an monthly restart cycle, archiving every 6 month and
    post-processing every 12 months.
              """
[scheduling]
    initial cycle point = 2010-01  # Note: '201001' is not permitted.
    final cycle point = 2012-12
    [[dependencies]]
        [[[R1]]]
            graph = """
                prep => model
            """
        [[[P1M]]]
            # sequence anchored on Januray 1st, 2010)
            graph = """
                model[-P1M] => model
                    """
        [[[P6M]]]
            # every sixth month, same anniversary date and anchor.
            graph = """
                model => archive
                    """
        [[[P12M]]]
            # every twelve month, same anniversary date and anchor.
            graph = """
                model => post
                    """
[visualization]
    initial cycle point = 2010-01
    final cycle point = 2012-12
    [[node attributes]]
        prep    =  "style=filled", "fillcolor=orange", "shape=circle"	
        model   =  "style=filled", "fillcolor=lightblue"	
        post    =  "style=filled", "fillcolor=seagreen2"	
        archive =  "style=filled", "fillcolor=red"	
