[cylc]
    [[parameters]]
        pets = cats, dogs, fish
        run = 0..2
[scheduling]
    [[dependencies]]
        graph = """
    pre => init<run>
    get<run,pets> => feed<run,pets>
    get<run,pets-1> => get<run,pets>
    feed<run,pets-1> => feed<run,pets>
    init<run> => get<run,pets=cats>
    feed<run,pets=fish> => post
    get<run=1,pets=dogs> => boo
            """

[runtime]
    [[pre]]
    [[post]]
    [[RUN<run>]]
    [[init<run>]]
        inherit = RUN<run>
    [[get<run, pets>]]
        inherit = RUN<run>
    [[feed<run, pets>]]
        inherit = RUN<run>
    [[get<run=1,pets>]]

[visualization]
    use node color for edges = True
    [[node attributes]]
        RUN<run=0> = "color=red"
        RUN<run=1> = "color=blue"
        get<run,pets=cats> = "style=filled", "fillcolor=LemonChiffon1"
        get<run,pets=dogs> = "style=filled", "fillcolor=LemonChiffon2"
        get<run,pets=fish> = "style=filled", "fillcolor=LemonChiffon3"
        feed<run,pets=cats> = "style=filled", "fillcolor=thistle1"
        feed<run,pets=dogs> = "style=filled", "fillcolor=thistle2"
        feed<run,pets=fish> = "style=filled", "fillcolor=thistle3"
        pre = "style=filled", "fillcolor=gray"
        post = "style=filled", "fillcolor=gray"
        boo = "style=filled", "fillcolor=yellow"
