title = 'inheritance example'

description = """
This suite illustrates several tiers of task runtime inheritance.
To see the result, use 'cylc get-config' or examine task job scripts
generated with 'cylc jobscript'."""

[scheduling]
    initial cycle point = 20110101T00
    final cycle point = 20110103T00
    [[dependencies]]
        [[[T00]]]
            graph = """
                foo => OPS
                OPS:finish-all & OPS:succeed-any => VAR
                VAR:succeed-all => BAZ
                atovs => atovs_post
                """
[runtime]
    [[root]]
        [[[environment]]]
            ROOT = "the quick brown fox"
    [[GEN]]
        [[[environment]]]
            GEN_A = gen_a
            GEN_B = gen_b
    [[OPS]]
        inherit = GEN
        script = "echo run-OPS.sh; sleep 10"
        [[[environment]]]
            GEN_B = ops_b
    [[VAR]]
        inherit = GEN
        script = "echo run-VAR.sh; sleep 10"
        [[[environment]]]
            GEN_B = var_b
    [[atovs]]
        description = "OPS ATOVS obs processing"
        inherit = OPS
        [[[environment]]]
            OPS_DATA = atovs
    [[aircraft]]
        description = "OPS aircraft obs processing"
        inherit = OPS
        script = "echo HELLO from $CYLC_TASK_ID; sleep 5; echo ABORT; /bin/false"
        [[[environment]]]
            OPS_DATA = aircraft
    [[ConLS]]
        description = "runs VAR ConfigureLS"
        inherit = VAR
        [[[environment]]]
            VAR_PROG = ConfigureLS
    [[AnPF]]
        description = "runs VAR AnalysePF"
        inherit = VAR
        script = "echo run-VAR.sh --apf; sleep 10"
        [[[environment]]]
            VAR_PROG = AnalysePF
    [[BAZ]]
    [[bar1,bar2]]
        inherit = BAZ
        description = "member of BAZ"
    [[atovs_post]]
        inherit = OPS
        description = "OPS ATOVS postprocessing"

    [[prepobs]]
        description = "obs preprocessing"

[visualization]
    collapsed families = OPS, VAR, BAZ
