[cylc]
    UTC mode = True
    [[reference test]]
        live mode suite timeout = PT3M
[scheduling]
    initial cycle point = 1000W011
    final cycle point = 1000W064
    [[dependencies]]
        [[[ P1W ]]]
            graph = "foo[-P1W] => foo => bar"
        [[[ P3W ]]]
            graph = "baz[-P3W] => baz => qux"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 1000W011
    final cycle point = 1000W064
