[cylc]
   [[reference test]]
       required run mode = live
       live mode suite timeout = 0.5 # minutes
[scheduling]
    [[dependencies]]
        graph = "foo => bar"
[runtime]
    [[foo]]
        script = cylc ping $CYLC_SUITE_NAME
    [[bar]]
        script = [[ ! $(cylc ping $CYLC_SUITE_NAME-non-existant) ]]
