[cylc]
    cycle point format = "%Y%m%dT%H"
   [[reference test]]
       required run mode = live
       live mode suite timeout = PT90S
[scheduling]
    initial cycle time = 20100101T00
    final cycle time = 20100102T00
    [[dependencies]]
        [[[R1]]]
            graph = "start => a"
        [[[T00, T06]]]
            graph = "c[-PT6H] => a => b => c"
        [[[T18]]]
            graph = "c"
[runtime]
    [[a,c,start]]
        script = "sleep 1"
    [[b]]
        script = "cylc reload $CYLC_SUITE_REG_NAME -f; sleep 5"
