[cylc]
    [[events]]
        abort on inactivity = False
        abort on stalled = True
        inactivity handler = cylc release '%(suite)s'
        inactivity = PT5S
[scheduling]
    [[dependencies]]
        graph = t1
[runtime]
    [[t1]]
        script = true
