[cylc]
    [[reference test]]
        required run mode = live
        live mode suite timeout = 1.0 # minutes
[scheduling]
    initial cycle time = 2010010100
    final cycle time   = 2010010200
    [[dependencies]]
        [[[Daily(2010010100,1)]]]
            graph = "daily"
        [[[Monthly(2010010100,1)]]]
            graph = "monthly"
        [[[Yearly(2010010100,1)]]]
            graph = "yearly"
[runtime]
    [[root]]
        script = "true"
