#!jinja2

title = "hold family test suite"

description = """One task that selectively holds tasks in the first cycle
point of the suite."""

[cylc]
    UTC mode = True
    [[reference test]]
        live mode suite timeout = PT1M
[scheduling]
    initial cycle point = 20141009T00
    [[dependencies]]
        [[[R1]]]
            graph = holdrelease => foo & stop
        [[[P1D]]]
            graph = """
                foo => bar
            """
[runtime]
    [[holdrelease]]
        script = """
sleep 5
cylc hold "$CYLC_SUITE_NAME" '{{HOLD_MATCH}}.20141009T0000Z'
sleep 5
"""
    [[STUFF]]
    [[STOP]]
    [[foo,bar]]
        inherit = STUFF
        script = true
    [[stop]]
        inherit = STOP
        script = sleep 5; cylc stop $CYLC_SUITE_NAME
