[cylc]
    UTC mode = True
    cycle point format = %Y
    [[reference test]]
        live mode suite timeout = PT1M

[scheduling]
    initial cycle point = 2010
    final cycle point   = 2011
    [[dependencies]]
        [[[P1Y]]]
            graph = foo

[runtime]
    [[foo]]
        script = true
