title = "Job script: no directives test for SLURM"

[cylc]

[scheduling]
    [[dependencies]]
        graph = foo

[runtime]
    [[foo]]
        script = echo hello
        [[[job]]]
            batch system = slurm
