title = "A complex task family example"

description = "Task families with family and extra-family dependencies."

[scheduling]
    initial cycle point = 20110101T12
    final cycle point = 20110103T12
    [[special tasks]]
        sequential = a  # (just to slow the suite down a bit)
    [[dependencies]]
        [[[T12]]]
            graph = """
# full family triggers:
a => FAMX
FAMX:succeed-all => B => FAMY
FAMY:succeed-all => C
# internal dependence among family members:
x1 => x2
# member of one family to member of another:
x2 => y2
# extra-family dependence:
y1 => D => y2
E => x1 => F
x1 | x2 => G
y1 | H => I"""

[runtime]
    [[FAMX, FAMY]]
    [[x1,x2]]
        inherit = FAMX
    [[y1,y2]]
        inherit = FAMY

[visualization]
    collapsed families = FAMX, FAMY
    use node color for edges = False
    [[node attributes]]
        FAMx = 'color=red', 'shape=ellipse'
        FAMY = 'color=blue', 'shape=ellipse'
