[cylc]
    UTC mode = True
    [[reference test]]
        live mode suite timeout = PT3M
[scheduling]
    initial cycle point = 2005023T12
    final cycle point = 2008001T11
    [[dependencies]]
        [[[ P1Y ]]]
            graph = "foo[-P1Y] => foo => bar"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 2005023T12
    final cycle point = 2008001T11
