[scheduling]
    initial cycle point = 20120909T00
    final cycle time   = 20120913T00

    [[dependencies]]
        [[[R1/T12]]]
            graph = """
                waz1 => waz2
                cfoo => foo
            """
        [[[T12]]]
            graph = """
                foo[-P1D] => foo => bar
            """
        [[[T12/P2D]]]
            graph = """
                foo => BAZ
            """
[visualization]
    initial cycle point = 20120909T00
    final cycle time   = 20120913T00
    [[node attributes]]
        foo = "color=blue"
        bar = "color=red"
        BAZ = "style=filled", "color=skyblue"
