[cylc]
   [[reference test]]
       required run mode = live
       live mode suite timeout = 0.5 # minutes
       allow task failures = True
       expected task failures = foo.1

[scheduling]
    [[dependencies]]
        graph = """foo:submit-fail => bar
                   bar => !foo"""
[runtime]
    [[foo]]
        script = "true"
        [[[remote]]]
            host=idontexist
    [[bar]]
        script = "true"
