[cylc]
   [[reference test]]
       required run mode = live
       live mode suite timeout = 0.5 # minutes
       allow task failures = True
       expected task failures = b.1

[scheduling]
    [[dependencies]]
        graph = """FAM:fail-any => foo
                   foo => !FAM"""
[runtime]
    [[FAM]]
        script = "true"
    [[a,c]]
        inherit = FAM
    [[b]]
        inherit = FAM
        script = "false"
    [[foo]]
        script = "true"
