#!jinja2
[cylc]
    UTC mode=True
    [[reference test]]
        expected task failures = t1.19700101T0000Z
        live mode suite timeout = PT1M
[scheduling]
    initial cycle point=1970
    final cycle point=1970
    [[dependencies]]
        [[[P1Y]]]
            graph=t1
[runtime]
    [[t1]]
        script=printenv CYLC_TEST_VAR_FOO
