#!Jinja2

{% set HOST = environ['CYLC_TEST_TASK_HOST'] %}
{% set OWNER = environ['CYLC_TEST_TASK_OWNER'] %}

# whether or not ssh messaging is used back from the remote host is
# determined by the [hosts] section in site/user config files.

[cylc]
    [[reference test]]
        live mode suite timeout = 0.5
        suite shutdown event handler = cleanup.sh
[scheduling]
    [[dependencies]]
        graph = "foo => bar"
[runtime]
    [[foo]]
        script = "/bin/true" # fast
        [[[remote]]]
            host = {{ HOST }}
            owner = {{ OWNER }}
    [[bar]]
        script = "/bin/true"
        [[[remote]]]
            host = {{ HOST }}
