# The following themes come with cylc.
# Define custom themes in $HOME/.cylc/gcylc.rc.

[themes]
    [[default]]
        # Default theme rationale:
        # * States representing something that happened external to cylc
        # are solid; those representing a task proxy states purely
        # internal to cylc are hollow. Thus pre-submitted states, and
        # the retry state, are hollow.
        # * Normal states are pleasant shades of grey, blue, or green.
        # * Failure states are bright red ('submit-failed' is hollow).
        # * The 'running' state is solid green. The 'queued',
        #   'ready', and 'submitted' states are different
        #   shades of yellow-to-green to indicate "almost running".
        # * The operator-held state is pink, to indicate an abnormal but
        #   non-threatening state.
        # * The 'succeeded' state is uninteresting, hence grey.
        # * The 'retry' states are hollow (pre-running) orange (almost red)
        #   as a failure has occurred but we're doing something about it.
        # * The 'waiting' state is light blue.
        defaults  = "color=#bbbbbb", "style=unfilled", "fontcolor=black"
        waiting   = "color=#88c6ff"
        held      = "color=#fe83ff"
        queued    = "color=#dcd901"
        ready     = "color=#a08f49"
        expired   = "color=#000000"
        submitted = "color=#a1cf25", "style=filled"
        submit-failed = "color=#ff007e", "style=filled", "fontcolor=white"
        running   = "color=#00c140", "style=filled"
        succeeded = "color=#ada5a5", "style=filled"
        failed    = "color=#ff0000", "style=filled", "fontcolor=white"
        retrying  = "color=#ff7e00"
        submit-retrying  = "color=#fa7e00"
        runahead   = "color=#b35bfd"

    [[solid]]
        # a solid version of the default theme
        inherit = "default"
        defaults = "color=#bbbbbb", "style=filled", "fontcolor=black"
        expired  = "color=#000000", "fontcolor=white"

    [[high-contrast]]
        inherit = "default"
        defaults  = "color=#bbbbbb", "style=filled", "fontcolor=white"
        waiting   = "color=#008ed3"
        held      = "color=#fc00ff"
        queued    = "color=#e6c522", "fontcolor=black"
        ready     = "color=#9c8320"
        expired   = "color=#000000", "fontcolor=white"
        submitted = "color=#76a200", "fontcolor=black"
        submit-failed = "color=#ff6f00"
        running   = "color=#008e27"
        succeeded = "color=#555454"
        failed    = "color=#ff0035"
        retrying  = "color=#ff6282"
        submit-retrying  = "color=#ffaf6f"
        runahead   = "color=#8000c0"

    [[color-blind]]
         # Ten colors that are apparently distinguishable by people with
         # most forms of color-blindness:
         #   http://safecolors.rigdenage.com/colourchoice.html
         # Can also use style=filled/unfilled to help discrimate states.
        inherit = "default"
        defaults  = "color=#bbbbbb", "style=filled", "fontcolor=black"
        waiting   = "color=#00ccff" # blue
        queued    = "color=#336600" # moss green
        ready     = "color=#d2800a",  "style=unfilled"
        expired   = "color=#000000", "fontcolor=white"
        submitted = "color=#ff9933" # orange
        submit-failed = "color=#ff3333", "style=unfilled"
        running   = "color=#66ff33" # lime green
        succeeded = "color=#cccccc" # grey
        failed    = "color=#ff3333" # red
        held      = "color=#9900cc" # purple
        retrying = "color=#009999" # turquoise
        submit-retrying = "color=#007777" # turquoise2
        runahead   = "color=#000000"
