[cylc]
    cycle point format = %Y
    [[reference test]]
        live mode suite timeout = PT1M
        expected task failures = get_obs.2010
[scheduling]
    initial cycle point = 2010
    final cycle point = 2013
    max active cycle points = 4
    spawn to max active cycle points = True
    [[dependencies]]
        [[[P1Y]]]
            graph = get_obs => proc_obs
        [[[R1/P0Y]]]
            graph = proc_obs => cleanup
[runtime]
    [[get_obs]]
        script = """
((CYLC_TASK_CYCLE_POINT == CYLC_SUITE_INITIAL_CYCLE_POINT)) && false"""
    [[cleanup]]
        script = cylc reset -s succeeded $CYLC_SUITE_NAME get_obs:failed
