#!jinja2
{%- set TEST_DIR = environ['TEST_DIR'] %}
[cylc]
    UTC mode = True
    [[events]]
        timeout handler = shutdown_this_suite_hook
        timeout = PT3M
[scheduling]
    initial cycle time = 20130923T00
    final cycle time   = 20130923T00
    [[dependencies]]
        [[[R1]]]
            graph = """
                succeeded_task => shutdown
                shutdown => output_states
                output_states => finish
            """
[runtime]
    [[succeeded_task]]
        description = "Succeeded task (runs before restart)"
        script = """
            sleep 1
        """
{% include 'suite-runtime-restart.rc' %}
