[cylc]
   [[reference test]]
       required run mode = live
       live mode suite timeout = 1.0 # minutes
[scheduling]
    initial cycle time = 2013010100
    final cycle time = 2013010200
    [[dependencies]]
        [[[0,6,12,18]]]
            graph = "foo[T-6] => foo"
[runtime]
    [[foo]]
        script = "true"
