title = Authentication test suite.
description = Stalls when the first task fails.
[cylc]
    [[events]]
        timeout = PT30S
        abort on timeout = True
[scheduling]
    [[dependencies]]
        graph = foo => bar
[runtime]
    [[foo]]
        script = /bin/false
    [[bar]]
        script = /bin/true
