
[cylc]
    cycle point format = %Y%m%dT%H
    [[reference test]]
        required run mode = live
        live mode suite timeout = PT1M

[scheduling]
    initial cycle point = 20120101T00
    final cycle point = 20120101T18
    [[dependencies]]
        [[[PT6H]]]
            graph = a[+PT6H] => b
        [[[R/+PT6H/PT6H]]]
            graph = a
[runtime]
    [[a,b]]
        script = "/bin/true" # fast
