[cylc]
    [[reference test]]
        live mode suite timeout = PT3M
[scheduling]
    initial cycle point = 1
    final cycle point = +P15  # = 16
    runahead limit = P12
    cycling mode = integer
    [[special tasks]]
        sequential = seq
    [[dependencies]]
        [[[ R1 ]]]
            graph = wibble
        [[[ R1/+P6 ]]]
            graph = wibble[^] => wobble
        [[[ R1/P0 ]]]
            graph = wobble[^+P6] => wubble
        [[[ P3 ]]]
            graph = """
             seq => foo        # sequential task
        foo[-P3] => foo        # prev instance trigger
             foo => bar        # plain trigger
        woo[+P1] => foo        # prev cycle
        woo[+P1] => bar
        woo[-P1] => foo        # next cycle

        foo[-P3]:out1 => on_toast # message outputs
                    """
        [[[ +P1/P3 ]]]
            graph = woo
        [[[ R/+P2/P3 ]]]
            graph = woo
        [[[ R/7/P6 ]]]
            graph = """
         foo[-P3] => qux       # every second cycle
                    """
        [[[ R1/$ ]]]
            graph = baz
        [[[ R/P4!8 ]]]
            graph = ning
        [[[ R1/^ ]]]
            graph = nang
        [[[ R/+P1/P6!14 ]]]
            graph = nong
[runtime]
    [[foo]]
        script = """
sleep 5
cylc task message "the cheese is ready"
sleep 5
touch typing
                            """
        [[[outputs]]]
            out1 = "the cheese is ready"

[visualization]
    default node attributes = "style=filled"
    initial cycle point = 1
    final cycle point = 16
    [[node attributes]]
        foo = "fillcolor=slateblue"
        woo = "fillcolor=limegreen"
        bar = "fillcolor=orange"
        seq = "fillcolor=navajowhite"
        on_toast = "fillcolor=beige"
        qux = "fillcolor=orangered"
        wibble = "fillcolor=violet"
        wobble = "fillcolor=darkviolet"
        wubble = "fillcolor=mediumvioletred"
