title = "Task polling example suite."

description = """
Demos job submission and execution polling, for the poll task
communication method, and more generally. View the suite event log 
as the suite runs, filtering for the word 'poll'.
              """ 

[scheduling]
    [[dependencies]]
            graph = "foo & bar"
[runtime]
    [[foo]]
        # no polling for this task except with
        #  "task communication method = poll"
    [[bar]]
        script = "sleep 60"
        [[[job]]]
            batch system = at
            # stay 'submitted' for up to a minute:
            batch submit command template = at now + 1 minutes
            # configure submission and execution polling:
            submission polling intervals = PT6S
            execution polling intervals = PT12S, 2*PT18S, PT6S
