[cylc]
   [[reference test]]
       required run mode = live
       live mode suite timeout = 1.0 # minutes
[scheduling]
    [[dependencies]]
        graph = a:start => b
[runtime]
    [[a]]
        script = sleep 20
    [[b]]
        script = cylc poll "$CYLC_SUITE_REG_NAME" 'a' '1'
