[cylc]
   [[reference test]]
       required run mode = live
       live mode suite timeout = PT1M
[scheduling]
    [[dependencies]]
        graph = "foo"
[runtime]
    [[foo]]
        script = cylc scan | grep "^${CYLC_SUITE_NAME} "
