[cylc]
   UTC mode = True
   [[reference test]]
       live mode suite timeout = PT1M
       required run mode = live
[scheduling]
    initial cycle point = 2000
    final cycle point = 2005
    [[dependencies]]
        [[[P1Y]]]
            graph = foo => bar
[runtime]
    [[bar, foo]]
        script = printenv CYLC_TEST_VAR
