#!jinja2
[cylc]
   [[reference test]]
       required run mode = live
       live mode suite timeout = PT1M

[scheduling]
    [[dependencies]]
        graph = t1

[runtime]
    [[t1]]
        script = test "${CYLC_TASK_SUBMIT_NUMBER}" -eq 2
        [[[job]]]
            execution retry delays = P0Y
{% if CYLC_TEST_HOST is defined %}
        [[[remote]]]
            host = {{CYLC_TEST_HOST}}
{% endif %}
