[scheduling]
    [[dependencies]]
        graph = "foo => bar => baz => qux => bah => hum => bug => reg => exp"
[runtime]
    [[FAM1]]
        pre-script = echo pre
        [[[environment]]]
            MESSAGE="hello"
    [[FAM2]]
        post-script = echo post
        [[[environment]]]
            MESSAGE="goodbye"
    [[FAM3]]
        pre-script = echo pre2
    [[FAM4]]
        script = echo $MESSAGE
    [[FAM5]]
        script = echo new $MESSAGE
    [[foo]]
        inherit=FAM1
    [[bar]]
        inherit=FAM1,FAM2
    [[baz]]
        inherit=FAM1
        [[[environment]]]
            MESSAGE="baz"
    [[qux]]
        inherit=FAM1
        pre-script = echo qux
    [[bah]]
        inherit=FAM1,FAM3
    [[hum]]
        inherit=FAM2
        post-script = echo hum
    [[bug]]
        inherit=FAM4
    [[reg]]
        inherit=FAM4
        script = echo reg
    [[exp]]
        inherit=FAM4,FAM5
