
title = "hold/release test suite"

description = """One task that holds then releases the suite, with
short sleeps to make the effect on the downstream task obvious in the
GUI (waiting to held to waiting)"""

# ref: bug-fix GitHub Pull Request #843 (5412d01)

[cylc]
    cycle point format = %Y%m%dT%H
    [[reference test]]
        live mode suite timeout = PT30S

[scheduling]
    initial cycle time = 20140101T00
    final cycle time   = 20140101T00
    [[dependencies]]
        [[[R1]]]
            graph = "holdrelease => foo & bar"
        [[[T00, T06]]]
            graph = "bar"
[runtime]
    [[holdrelease]]
        script = """
sleep 5
cylc hold $CYLC_SUITE_NAME
sleep 5
cylc release $CYLC_SUITE_NAME
sleep 5"""
    [[foo,bar]]
        script = true
