
title = "job script cycling test"

description = """Test the output of cycling environment variables"""

[cylc]
    cycle point time zone = +13
    [[reference test]]
        live mode suite timeout = PT0.5M
        dummy mode suite timeout = PT0.5M
        simulation mode suite timeout = PT0.5M

[scheduling]
    initial cycle point = 20140201T05
    final cycle point = 20140204T11+12
    [[dependencies]]
        [[[T00]]]
            graph = "foo"
[runtime]
    [[foo]]
        init-script = echo HELLO FROM INIT-SCRIPT
        pre-script = echo HELLO FROM PRE-SCRIPT
        script = echo HELLO FROM SCRIPT
        post-script = echo HELLO FROM POST-SCRIPT
        [[[remote]]]
            # force use of ssh job submission to disconnect local tasks
            # from the submitting suite environment.
            host = localhost
