[cylc]
   [[reference test]]
       live mode suite timeout = PT1M
       required run mode = live
[scheduling]
    [[dependencies]]
        graph=foo&bar

[runtime]
    [[foo,bar]]
        script=true
