[cylc]
    UTC mode = True

[scheduling]
    initial cycle point = 20100808T06
    final cycle point =
    [[dependencies]]
        [[[T06]]]
            graph = foo
[runtime]
    [[foo]]
        script = sleep 5

