[cylc]
    [[events]]
        timeout = PT1M
        abort on timeout = True
[scheduling]
    [[dependencies]]
        graph = foo => bar & baz
[runtime]
    [[bar, baz]]
    [[foo]]
        script = /bin/false
