[scheduling]
    initial cycle point = 2020
    final cycle point = 2030
    [[dependencies]]
        [[[P1Y]]]
            graph = foo
[runtime]
    [[foo]]
        script = """
echo this
echo that
"""
