title = Suite-1000-x3
description = A suite of 1000 tasks per cycle
[cylc]
    UTC mode = True # Ignore DST
    [[parameters]]
        i = 0..9
        j = 0..9
        k = 0..0
        l = 0..9 # Don't change l
[scheduling]
    initial cycle point = 20130101T00
    final cycle point = 20130103T00
    runahead limit = PT12H
    [[dependencies]]
        [[[T00]]]
            graph="""
        t<i,j,k,l=9>[-P1D] => t<i,j,k,l=0>
        t<i,j,k,l=0> => t<i,j,k,l=1> => t<i,j,k,l=2> => t<i,j,k,l=3>
        t<i,j,k,l=3> => t<i,j,k,l=4> => t<i,j,k,l=5> => t<i,j,k,l=6>
        t<i,j,k,l=6> => t<i,j,k,l=7> => t<i,j,k,l=8> => t<i,j,k,l=9>"""
[runtime]
    [[root]]
        script = sleep 1
        [[[job]]]
            batch system = at
        [[[events]]]
            succeeded handler = true
            failed handler = true
            retry handler = true
            submission failed handler = true
            submission timeout handler = true
            execution timeout handler = true
            execution timeout = PT6M
            submission timeout = PT1M
    [[t<i,j,k,l>]]
