#!Jinja2

{% set START = "20100808T00" %}
{% set STOP  = "20100812T00" %}

[scheduling]
    initial cycle point = {{ START }}
    final cycle time   = {{ STOP }}
    [[dependencies]]
        [[[R1]]]
            graph = """
                cfoo => foo
            """
        [[[PT12H]]]
            graph = """
               foo[-PT12H] => foo => bar
                    """
        [[[P2D]]]
            graph = """
                foo => baz
                    """
[visualization]
    initial cycle point = {{ START }}
    final cycle time   = {{ STOP }}
    [[node attributes]]
        root = "style=filled", "fillcolor=yellow"
        bar = "style=filled", "fillcolor=green"
        baz = "style=filled", "fillcolor=red"
