#!Jinja2

title = "Jinja2 command line set test"
description = """A couple of Jinja2 variables must be set on the command
line or else this test will fail to validate: TASKNAME and STEP. Also,
to pass the test run their values must be 'foo' and '2', respectively."""

[cylc]
    [[reference test]]
        required run mode = live
        live mode suite timeout = 0.5

[scheduling]
    initial cycle time = 2010
    final cycle time = 2012
    [[dependencies]]
        [[[ Yearly(2010,{{STEP}}) ]]]
            graph = {{TASKNAME}}
[runtime]
    [[ {{TASKNAME}} ]]
        script = "sleep 1"
