[cylc]
    UTC mode = true
[scheduling]
    initial cycle point = 20160101T00Z
    final cycle point = 20160102T00Z
    [[dependencies]]
        [[[R1]]]
            graph = foo
        [[[R1/^]]]
            graph = bar
        [[[R1/^+PT1H]]]
            graph = baz
        [[[R1/$-PT1H]]]
            graph = boo
        [[[R1/$]]]
            graph = foo[^] & bar[^] & baz[^+PT1H] & boo[^+PT23H] => bot
[runtime]
    [[ root ]]
        script = echo "success"
