
title = "A test with nested include-files"

[scheduling]
    initial cycle time = 20130101T00
    final cycle time = 20130101T00
    [[dependencies]]
        [[[R1]]]
            graph = foo => bar
        [[[T00]]]
            graph = bar
[runtime]
    [[root]]
        script = "echo Hello World"

[visualization]
    default node attributes = "style=filled"
