[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20140101
    final cycle point = 20140104
    [[dependencies]]
        graph = "cold_foo"
        [[[ 0 ]]]
            graph = "foo_midnight[T-24] & cold_foo => foo_midnight"
        [[[ 6 ]]]
            graph = "foo_dawn[T-24] & cold_foo => foo_dawn"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 20140101
    final cycle point = 2014010206
