[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20140101
    final cycle point = 20140102T12
    [[dependencies]]
        [[[ R1 ]]]
            graph = "cold_foo => foo"
        [[[ PT12H ]]]
            graph = "foo[-PT12H] => foo"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 20140101
    final cycle point = 20140102T12
