
title = "test submit and submit-fail triggers."

[scheduling]
    [[dependencies]]
        graph = """
foo:submit-fail => bar
bar:submit => baz
                """

[runtime]
    [[root]]
        script = "sleep 60"
    [[foo]]
        [[[job]]]
            batch system = "FAIL"
