#!Jinja2

title = "Jinja2 example: use of defaults and external input"

description = """
The template variable N_MEMBERS can be set on the command line with
--set or --set-file=FILE; but if not a default values is supplied.""" 

{% set N_MEMBERS = N_MEMBERS | default( 3 ) | int %}

[scheduling]
    initial cycle point = 20100808T00
    final cycle time   = 20100816T00
    [[dependencies]]
        [[[T00]]]
            graph = """foo => ENS
                 ENS:succeed-all => bar"""
[runtime]
    [[ENS]]
{% for I in range( 0, N_MEMBERS ) %}
    [[ mem_{{ I }} ]]
        inherit = ENS
{% endfor %}
