2013/10/06 23:32:59 INFO - Thread-2 start (Event Handler Submission)
2013/10/06 23:32:59 INFO - Thread-3 start (Poll and Kill Command Submission)
2013/10/06 23:32:59 INFO - port:7766
2013/10/06 23:32:59 INFO - Suite starting at 2013-10-06 23:32:59.126325
2013/10/06 23:32:59 INFO - Log event clock: real time
2013/10/06 23:32:59 INFO - Run mode: live
2013/10/06 23:32:59 INFO - Initial point: 1
2013/10/06 23:32:59 INFO - Final point: 1
2013/10/06 23:32:59 INFO - Thread-4 start (Job Submission)
2013/10/06 23:32:59 INFO - Thread-5 start (Request Handling)
2013/10/06 23:32:59 INFO - [hello.1] -(setting:submitting)
2013/10/06 23:32:59 INFO - [hello.1] -triggered off []
2013/10/06 23:33:00 INFO - [hello.1] -(current:submitting)> hello.1 submitting now
2013/10/06 23:33:01 INFO - [hello.1] -(current:submitting)> hello.1 submission succeeded
2013/10/06 23:33:01 INFO - [hello.1] -(setting:submitted)
2013/10/06 23:33:01 INFO - [hello.1] -(current:submitted)> hello.1 submit_method_id=2177
2013/10/06 23:33:01 INFO - [hello.1] -(current:submitted)> hello.1 started at 2013-10-06T23:33:00
2013/10/06 23:33:01 INFO - [hello.1] -(setting:running)
2013/10/06 23:33:01 INFO - [hello.1] -(current:running)> hello.1 succeeded at 2013-10-06T23:33:00
2013/10/06 23:33:01 INFO - [hello.1] -(setting:succeeded)
2013/10/06 23:33:01 INFO - [goodbye_2.1] -(setting:submitting)
2013/10/06 23:33:01 INFO - [goodbye_1.1] -(setting:submitting)
2013/10/06 23:33:01 INFO - [goodbye_0.1] -(setting:submitting)
2013/10/06 23:33:01 INFO - [goodbye_2.1] -triggered off ['hello.1']
2013/10/06 23:33:01 INFO - [goodbye_1.1] -triggered off ['hello.1']
2013/10/06 23:33:01 INFO - [goodbye_0.1] -triggered off ['hello.1']
2013/10/06 23:33:03 INFO - [goodbye_2.1] -(current:submitting)> goodbye_2.1 submitting now
2013/10/06 23:33:03 INFO - [goodbye_2.1] -(current:submitting)> goodbye_2.1 submission succeeded
2013/10/06 23:33:03 INFO - [goodbye_2.1] -(setting:submitted)
2013/10/06 23:33:03 INFO - [goodbye_2.1] -(current:submitted)> goodbye_2.1 submit_method_id=2221
2013/10/06 23:33:03 INFO - [goodbye_2.1] -(current:submitted)> goodbye_2.1 started at 2013-10-06T23:33:02
2013/10/06 23:33:03 INFO - [goodbye_2.1] -(setting:running)
2013/10/06 23:33:03 INFO - [goodbye_2.1] -(current:running)> goodbye_2.1 succeeded at 2013-10-06T23:33:02
2013/10/06 23:33:03 INFO - [goodbye_2.1] -(setting:succeeded)
2013/10/06 23:33:03 INFO - [goodbye_1.1] -(current:submitting)> goodbye_1.1 submitting now
2013/10/06 23:33:03 INFO - [goodbye_1.1] -(current:submitting)> goodbye_1.1 started at 2013-10-06T23:33:02
2013/10/06 23:33:03 INFO - [goodbye_1.1] -(setting:running)
2013/10/06 23:33:03 INFO - [goodbye_1.1] -(current:running)> goodbye_1.1 succeeded at 2013-10-06T23:33:02
2013/10/06 23:33:03 INFO - [goodbye_1.1] -(setting:succeeded)
2013/10/06 23:33:03 WARNING - [goodbye_1.1] -Assuming non-reported outputs were completed:
goodbye_1.1 submitted
2013/10/06 23:33:03 INFO - [goodbye_0.1] -(current:submitting)> goodbye_0.1 submitting now
2013/10/06 23:33:03 INFO - [goodbye_0.1] -(current:submitting)> goodbye_0.1 submission succeeded
2013/10/06 23:33:03 INFO - [goodbye_0.1] -(setting:submitted)
2013/10/06 23:33:03 INFO - [goodbye_0.1] -(current:submitted)> goodbye_0.1 submit_method_id=2271
2013/10/06 23:33:03 INFO - [goodbye_0.1] -(current:submitted)> goodbye_0.1 started at 2013-10-06T23:33:02
2013/10/06 23:33:03 INFO - [goodbye_0.1] -(setting:running)
2013/10/06 23:33:03 INFO - [goodbye_0.1] -(current:running)> goodbye_0.1 succeeded at 2013-10-06T23:33:02
2013/10/06 23:33:03 INFO - [goodbye_0.1] -(setting:succeeded)
2013/10/06 23:33:03 INFO - All non-cycling tasks have succeeded
2013/10/06 23:33:03 INFO - Suite shutting down at 2013-10-06 23:33:03.159824
