[cylc]
   [[reference test]]
       required run mode = live
       live mode suite timeout = 1.0 # minutes
[scheduling]
    initial cycle time = 201002
    final cycle time = 201008
    runahead limit = 24 # enforce sequential running
    [[dependencies]]
        [[[Monthly(201001,3)]]]
            graph = "a"
[runtime]
    [[a]]
        script = "sleep 1"
