#!jinja2
{%- set TEST_DIR = environ['TEST_DIR'] %}
[cylc]
    UTC mode = True
    [[events]]
        timeout handler = shutdown_this_suite_hook
        timeout = PT3M
[scheduling]
    initial cycle time = 20130923T00
    final cycle time   = 20130923T00
    [[dependencies]]
        [[[R1]]]
            graph = """
                running_task:start => shutdown
                shutdown => output_states
                output_states & running_task => finish
            """
[runtime]
    [[running_task]]
        description = "Running task (runs during restart)"
        script = """
            sleep 50
        """
{% include 'suite-runtime-restart.rc' %}
    [[shutdown]]
        post-script = """
            cylc shutdown --now $CYLC_SUITE_REG_NAME
            sleep 5
        """
