title = "family failure task event hook example"

description = """
Illustrates use of a task event hook script to clean up failed tasks."""

[scheduling]
    initial cycle point = 20110101T06
    final cycle point = 20110102T00
    [[dependencies]]
        [[[T00, T06, T12, T18]]]
            graph =  """
                a => FAM
                a[-PT6H] => a
                FAM:finish-all & FAM:succeed-any => b"""
[runtime]
    [[a]]
    [[b]]
    [[FAM]]
        [[[events]]]
            failed handler = failhook.sh
    [[m_foo]]
        inherit = FAM
        script = sleep 8
    [[m_bar]]
        inherit = FAM
        # This family member always fails
        script = "sleep 5; /bin/false"
    [[m_baz]]
        inherit = FAM
        script = sleep 10
