title = A suite to test 'cylc trigger --edit'.
description = """
A broken task will fail and cause the suite to abort on timeout, unless a
second task fixes and retriggers it with an edit-run."""
[cylc]
    [[events]]
        abort on timeout = True
        timeout = PT20S
[scheduling]
    [[dependencies]]
        graph = broken-task:fail => fixer-task
[runtime]
    [[broken-task]]
        script = /bin/false
    [[fixer-task]]
        script = """
cylc trigger --edit $CYLC_SUITE_NAME broken-task 1 << __END__
y
__END__"""
