title = test suite for deprecated cylc-6 message triggers
[cylc]
    UTC mode = True
    [[reference test]]
        live mode suite timeout = PT50S

[scheduling]
    initial cycle point = 20140801T00
    final cycle point = 20141201T00
    [[dependencies]]
        [[[P2M]]]
           graph = """
foo:x => bar
foo[-P2M]:y => baz
"""

[runtime]
    [[bar, baz]]
        script = true
    [[foo]]
        script = """
echo HELLO
MESSAGE_X="file 1 for $CYLC_TASK_CYCLE_POINT done"
MESSAGE_Y="file 2 for $(cylc cycle-point --offset P2M) done"
cylc message "${MESSAGE_X}" "${MESSAGE_Y}"
grep -q "CYLC_MESSAGE=.*|NORMAL|${MESSAGE_X}" "$0.status"
grep -q "CYLC_MESSAGE=.*|NORMAL|${MESSAGE_Y}" "$0.status"
"""
        [[[outputs]]]
            x = "file 1 for [] done"
            y = "file 2 for [P2M] done"
