[scheduling]
    initial cycle time = 2013122512
    final cycle time = 2013122512
    [[dependencies]]
        [[[ 12 ]]]
            graph = """
                (a[T-24]:fail | b[T-24]:fail | c[T-24]:fail) => d
                a & b & c  # Implied by implicit cycling now...
            """
[runtime]
    [[root]]
        script = sleep 1
