title = "Hello, Goodbye, Suicide"
[cylc]
    [[reference test]]
        required run mode = live
        live mode suite timeout = 1.0
        expected task failures = goodbye.1

[scheduling]
    [[dependencies]]
        graph = """hello => goodbye
           goodbye:fail => really_goodbye
           goodbye => !really_goodbye
           really_goodbye => !goodbye"""
[runtime]
    [[hello]]
        script = "sleep 1; echo Hello World!"
    [[goodbye]]
        script = """
sleep 1
echo Goodbye ... oops!
/bin/false # FAIL"""
