[cylc]
    cycle point time zone = Z
    [[events]]
        timeout = PT0.3M
        abort on timeout = True
[scheduling]
    runahead limit = PT18H
    initial cycle time = 20100101T00
    [[dependencies]]
        [[[PT6H]]]
            graph = "foo => bar"
        [[[T12]]]
            graph = "foo:fail => shutdown"
[runtime]
    [[foo]]
        script = false
    [[bar]]
        script = true
    [[shutdown]]
        script = """
sleep 5
cylc shutdown $CYLC_SUITE_REG_NAME"""
