[cylc]
    [[events]]
        timeout = 0.5
        timeout handler = cylc shutdown $CYLC_SUITE_REG_NAME --now
[scheduling]
    initial cycle time = 2010010100
    final cycle time = 2010010500
    [[dependencies]]
        [[[0]]]
            graph = "bar[T-12] => foo"
        [[[12]]]
            graph = "foo[T-12] => bar"
[runtime]
    [[foo]]
        script = sleep 5
    [[bar]]
        script = rm -rf $CYLC_SUITE_RUN_DIR
