[cylc]
    UTC mode = True
   [[reference test]]
       required run mode = live
       live mode suite timeout = PT0.5M

[scheduling]
    initial cycle point = 20100101T0000Z
    final cycle point = 20100125T0000Z
    [[dependencies]]
        [[[ R5/W-1/P1W ]]]
            graph = foo

[runtime]
    [[foo]]
        script = true
