title = "Hello and Goodbye World!"
[scheduling]
    [[dependencies]]
        graph = "hello => goodbye"
[runtime]
    [[hello]]
        script = "sleep 10; echo Hello World!"
    [[goodbye]]
        script = "sleep 10; echo Goodbye World!"
