[cylc]
   [[reference test]]
       required run mode = live
       live mode suite timeout = 1.0 # minutes
[scheduling]
    [[dependencies]]
        graph = "foo"
[runtime]
    [[foo]]
        script = "((CYLC_TASK_TRY_NUMBER >= 4)) || exit 1"
        # ( '|| exit 1' required to trigger error trap in bash 4.2.0)
        [[[job]]]
            execution retry delays = 3*0.1
