
[cylc]
    [[events]]
        abort on timeout = True
    [[reference test]]
        live mode suite timeout = 1.5

[scheduling]
    initial cycle time = 2014010100
    final cycle time = 2014010300
    runahead limit = 24
    [[dependencies]] 
        [[[0]]] 
            graph = "foo[T-13] => bar" 
        [[[3]]] 
            graph = "foo[T-3] => bar" 
        [[[11]]] 
            graph = "foo[T-8] => bar"
