[cylc]
    [[events]]
        timeout = PT1M
        abort on timeout = True
[scheduling]
    [[queues]]
        [[[default]]]
            limit = 1
    [[dependencies]]
        graph = m1 & m2
[runtime]
    [[m1, m2]]
        script = sleep 10
