[cylc]
    [[reference test]]
        live mode suite timeout = PT2M
[scheduling]
    initial cycle point = 20001231T0100
    final cycle point = 20010114
    [[dependencies]]
        [[[ P1D ]]]
            graph = "foo[-P1D] => foo => bar"
        [[[ P4D ]]]
            graph = "baz[-P4D] => baz => qux"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 20001231T0100
    final cycle point = 20010114
