#!jinja2
[scheduling]
    [[dependencies]]
        graph = foo
[runtime]
    [[foo]]
        script = echo 'It is a good test.'
{% if CYLC_TEST_HOST %}
        [[[remote]]]
            host = {{CYLC_TEST_HOST}}
{% endif %}
{% if CYLC_TEST_BATCH_SYS_NAME %}
        [[[job]]]
            batch system = {{CYLC_TEST_BATCH_SYS_NAME}}
{% if CYLC_TEST_BATCH_SYS_NAME == "loadleveler" %}
        [[[directives]]]
            job_type = serial
            notification = never
            wall_clock_limit = 120,60
{% if "CYLC_TEST_DIRECTIVES" in environ and environ["CYLC_TEST_DIRECTIVES"] %}
    {{environ["CYLC_TEST_DIRECTIVES"]}}
{% endif %}
{% endif %}
{% endif %}
