[scheduling]
    initial cycle point = 20100101T00
    [[dependencies]]
        [[[T00,T06,T12]]]
            graph = "foo"
[runtime]
    [[root]]
        [[[events]]]
            execution timeout = 3
