title = a minimal self-contained remote task suite

description = """
This runs one task on a remote host. To use it, install cylc (and Pyro
OR ssh key for ssh-based remote messaging) on the remote host specified
under [[[remote]]] below.  Remote cylc location, and ssh messaging, can
be set in host-specific sections of the cylc site/user config file."""

[scheduling]
    [[dependencies]]
        graph = "foo => bar"
[runtime]
    [[root]]
        script = """
echo HELLO from $CYLC_TASK_NAME on $(hostname)
sleep 10
echo GOODBYE from $CYLC_TASK_NAME on $(hostname)"""
        [[[remote]]]
            # the remote host name:
            host = wrh-1.hpcf.niwa.co.nz
