[cylc]
    [[reference test]]
        live mode suite timeout = 0.8
[scheduling]
    initial cycle time = 2010010100
    final cycle time = 2010010500
    [[dependencies]]
        [[[0, 6, 12, 18]]]
            graph = "a[T-6] & c[T-6] => a => stopper => c"
[runtime]
    [[a,c]]
        script = "true"
    [[stopper]]
        script = "cylc shutdown $CYLC_SUITE_REG_NAME 2010010100; sleep 5"
