[cylc]
UTC mode = true
[[reference test]]
required run mode = live
live mode suite timeout = PT1M # minutes
[scheduling]
initial cycle point = 20100101T03
[[dependencies]]
[[[R1/min(T00-PT20H,T06,T12,T18)]]]
graph = foo

[runtime]
[[foo]]
