[cylc]
   [[reference test]]
       required run mode = live
       live mode suite timeout = 1.0 # minutes
[scheduling]
    initial cycle time = 2010080800
    final cycle time = 2010080900
    [[dependencies]]
        [[[0]]]
            graph = "foo[T-24]:restart1 => foo"
[runtime]
    [[foo]]
        script = """
sleep 1
cylc message "restart files ready for $(cylc cycle-point --offset-hours=24 )"
sleep 1"""
        [[[outputs]]]
            restart1 = "restart files ready for [T+24]"
