[cylc]
    cycle point format = %Y
    [[events]]
        inactivity = PT1M
        abort on inactivity = True
[scheduling]
    initial cycle point = now
    final cycle point = +P1Y
    max active cycle points = 1
    [[special tasks]]
        clock-trigger = foo(P0Y)
    [[dependencies]]
        [[[P1Y]]]
            graph = """foo => bar
                bar[-P1Y] => bar""" # (or sequential bar)
[runtime]
    [[root]]
        script = /bin/true
