[cylc]
    UTC mode = True
    [[events]]
        timeout = PT5S
        abort on timeout = True
[scheduling]
    initial cycle point = 2001
    final cycle point = 2010
    [[dependencies]]
        [[[P1Y]]]
            graph = foo
[runtime]
    [[foo]]
        script=sleep 1
