title = "Two cycling tasks with inter-cycle dependence."
[cylc]
    cycle point time zone = Z
[scheduling]
    #runahead limit = 120
    initial cycle point = 20130808T00
    final cycle point = 20130812T00
    [[dependencies]]
        # Repeat at 00 and 12 UTC every day:
        [[[T00,T12]]]
            graph = "foo[-PT12H] => foo => bar"
[visualization]
    initial cycle point = 20130808T00
    final cycle point = 20130809T00
    [[node attributes]]
        foo = "color=red"
        bar = "color=blue"
