[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 2015-02-21T00
    final cycle point = 2015-02-21T18
    [[dependencies]]
        [[[PT6H]]]
            graph = foo
        [[[R1/P0D]]]
            graph = foo => stop
        [[[R/PT6H/-PT6H]]]
            graph = foo => nonstop
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 2015-02-21T00
    final cycle point = 2015-02-21T18
