[cylc]
    UTC mode = True
   [[reference test]]
      live mode suite timeout = PT2M
[scheduling]
    initial cycle point = 20140101
    [[dependencies]]
        [[[ R1 ]]]
            graph = "cold_foo"
        [[[ PT12H ]]]
            graph = "cold_foo[^] & foo[-PT12H] => foo"
        [[[ R1/+P4D ]]]
            graph = "stop => foo"
[runtime]
    [[root]]
        script = true
    [[stop]]
        script = "cylc shutdown $CYLC_SUITE_REG_NAME 20140107"
[visualization]
    initial cycle point = 20140101
    final cycle point = 20140110
