[scheduling]
    initial cycle point = 2010
    final cycle point = 2015
    [[dependencies]]
        [[[R1/0505T05+1300]]]
            # at the first occurring 5 May at 0500, UTC + 13 hours timezone
            graph = """
                cfoo => foo
            """
        [[[0505T05+1300/P1Y]]]
            # every 5 May at 0500, UTC + 13 hours timezone
            graph = """
                foo[-P1Y] => foo => bar
                    """
        [[[0505T05+1300/P2Y]]]
            # every second year, same anniversary date/time.
            graph = """
                foo => baz
                    """
[visualization]
    initial cycle point = 2010
    final cycle point = 2015
