title = "test suite for deprecated cylc-5 message triggers"

[cylc]
    [[reference test]]
        live mode suite timeout = 0.8 # minutes

[scheduling]
    initial cycle time = 201408
    final cycle time = 201412
    [[special tasks]]
    [[dependencies]]
        [[[Monthly(201408,2)]]]
          graph = """
            foo:x => bar
            foo[T-2]:y => baz
                  """
[runtime]
    [[foo]]
        script = """
sleep 2
TARGET_POINT=$CYLC_TASK_CYCLE_POINT
cylc message "uploaded file 1 for $TARGET_POINT"
sleep 2
TARGET_POINT=$(cylc cycletime --offset-months=2)
cylc message "uploaded file 2 for $TARGET_POINT"
"""
        [[[outputs]]]
            x = "uploaded file 1 for [T]"
            y = "uploaded file 2 for [T+2]"
