#!jinja2
[cylc]
    UTC mode = True # Ignore DST
    [[reference test]]
        live mode suite timeout = PT10M

[scheduling]
    [[dependencies]]
        graph="""foo => bar"""

[runtime]
{% if "CYLC_TEST_HOST" in environ %}
    [[root]]
        [[[remote]]]
            host = {{environ["CYLC_TEST_HOST"]}}
{% endif %}
    [[foo]]
        script="""
JOB_LOG_DIR=$(dirname $(dirname "$CYLC_TASK_LOG_ROOT"))
readlink "$JOB_LOG_DIR/NN" >>what-is-nn.txt
echo "$JOB_LOG_DIR/NN"
# bash 4.2.0 bug: ((VAR == VAL)) does not trigger 'set -e':
test "$CYLC_TASK_TRY_NUMBER" -eq "100"
"""
        [[[job]]]
            execution retry delays=99*PT0S
    [[bar]]
        script="""
printf "%02d\n" {1..100} | cmp -s ../foo/what-is-nn.txt -
"""
