[cylc]
   [[reference test]]
       required run mode = live
       live mode suite timeout = 1.0 # minutes

[scheduling]
    [[dependencies]]
        graph = """FAM:start-all => foo"""
[runtime]
    [[FAM]]
        script = "sleep 5"
    [[a,b,c]]
        inherit = FAM
    [[foo]]
        script = "true"
