
description = """
Tests triggering of asynchronous one-off, synchronous one-off
(start-up), and cycling tasks in one suite."""

[cylc]
   [[reference test]]
       required run mode = live
       live mode suite timeout = 0.5 # minutes

[scheduling]
    initial cycle time = 2010080800
    final cycle time = 2010080900
    [[special tasks]]
        start-up = sup
    [[dependencies]]
        graph = "x1 => x2"
        [[[0]]]
            graph = "x2 & sup => foo => bar"

[runtime]
    [[root]]
        script = "true" # fast
