#!Jinja2

# This suite uses a custom Jinja2 filter called "pad(len,fill)" to left
# pad integer values with a fill character out to some string length.

# The custom filter must be implemented as a Python function of the same
# name in a Python module of the same name, stored in a "Jinja2Filters"
# sub-directory under the suite definition directory or $HOME/.cylc/.

# The Jinja2 loop below generates the following graph lines:
#| graph = """
#|    foo => bar_01
#|    foo => bar_02
#|        ...
#|    foo => bar_10
#|         """

[scheduling]
    [[dependencies]]
        graph = """
{% for i in range(1,11) %}
        foo => bar_{{i|pad(2,'0')}}  # see Jinja2/pad.py
{% endfor %}"""
