#!Jinja2
[cylc]
   UTC mode = True
   [[reference test]]
       live mode suite timeout = PT1M
       required run mode = live
[scheduling]
    initial cycle point = 2016
    final cycle point = {{FINAL_CYCLE_POINT}}
    [[dependencies]]
        [[[P1Y]]]
            graph = foo
[runtime]
    [[foo]]
        script = {{COMMAND}}
