[cylc]
    UTC mode = true
    [[event hooks]]
        # Reset the pre-start foo inserted by the inserter task.
        stalled handler = cylc reset %(suite)s foo.20100101T1200Z -s ready
        abort on timeout = True
        timeout = PT2M
    [[reference test]]
        live mode suite timeout=PT2M

[scheduling]
    initial cycle point = 20100101T0000Z
    final cycle point   = 20100102T0000Z
    [[dependencies]]
        [[[T00, T06, T12, T18]]]
            graph = bar[-PT6H] => foo => bar
        [[[T18]]]
            graph = inserter => bar

[runtime]
    [[root]]
        script = sleep 5
    [[INSERT_FAM]]
    [[foo,bar]]
        inherit = INSERT_FAM
    [[inserter]]
        script = cylc insert $CYLC_SUITE_NAME INSERT_FAM.20100101T1200Z --stop-point=20100101T1200Z
