[cylc]
    UTC mode = True
[scheduling]
    initial cycle point = 20140101
    final cycle point = 20140102T12
    [[dependencies]]
        [[[ +P1D/PT6H ]]]
            graph = "bar => baz"
[runtime]
    [[root]]
        script = true
[visualization]
    initial cycle point = 20140101
    final cycle point = 20140102T12
