[cylc]
   [[reference test]]
       required run mode = live
       live mode suite timeout = PT15S

[scheduling]
    [[dependencies]]
        graph = foo

[runtime]
    [[foo]]
        script = true
        [[[job]]]
            batch system = at
