[cylc]
    UTC mode = True
    [[reference test]]
        required run mode = live
        live mode suite timeout = PT2M
[scheduling]
    [[dependencies]]
        graph = """
locker:lock => TRUES
TRUES:succeed-all => done
"""
[runtime]
    [[locker]]
        script = sleep 5; timeout 60 cylc-db-lock
        [[[outputs]]]
            lock = "[] I have locked the public database file"
    [[TRUES]]
        script = true
    [[t0, t1, t2, t3, t4, t5, t6, t7, t8, t9]]
        inherit = TRUES
    [[done]]
        script = """
while ! grep -F -q 'cylc-suite-public.db: recovered' "${CYLC_SUITE_LOG_DIR}/log"; do
    sleep 1
done
"""
