[cylc]
    UTC mode = True
    cycle point format = %Y
    [[reference test]]
        required run mode = live
        live mode suite timeout = PT30S

[scheduling]
    initial cycle point = 2020
    final cycle point = 2021
    [[dependencies]]
        [[[R1]]]
            graph = remover
        [[[P1Y]]]
            graph = foo => bar & baz
[runtime]
    [[remover]]
        script = """
# Remove first bar, after spawning it
# (it will run in the second cycle).
cylc remove $CYLC_SUITE_NAME bar.2020
# Remove first baz, without spawning it
# (so it will not run at all).
cylc remove --no-spawn $CYLC_SUITE_NAME baz.2020"""
    [[bar, baz]]
        script = true
