title = Generate log messages for various suite connection types.
[scheduling]
    [[dependencies]]
        graph = foo
[runtime]
    [[foo]]
        script = """
sleep 5
# COMMAND INTERFACE:
UUID=$(cylc hold --print-uuid $CYLC_SUITE_NAME 2>&1 > /dev/null)
# INFO INTERFACE:
cylc show --set-uuid=$UUID $CYLC_SUITE_NAME
# BROADCAST INTERFACE:
cylc broadcast --set-uuid=$UUID -d $CYLC_SUITE_NAME
# COMMAND INTERFACE:
cylc release --set-uuid=$UUID $CYLC_SUITE_NAME
sleep 5"""
