
[scheduling]
    initial cycle point = 20100808T00
    runahead limit = P1D
    [[dependencies]]
        [[[R1]]]
            graph = cold => ObSensitivity
        [[[PT6H]]]
            graph = """
( LongForecast[-PT6H] & LongForecast ) => ObSensitivity
Analysis[+PT12H] => ObSensitivity
Analysis
                """
