[cylc]
    UTC mode=True
    [[reference test]]
        allow task failures = True
        live mode suite timeout = PT1M
[scheduling]
    initial cycle point=2020
    final cycle point=2020
    [[dependencies]]
        [[[P1Y]]]
            graph = t1
[runtime]
    [[t1]]
        script=test "${CYLC_TASK_SUBMIT_NUMBER}" -gt 2
        [[[job]]]
            execution retry delays=2*PT0S
