title = "Task parameterisation example, two parameters."
[cylc]
    [[parameters]]
        r = 0..2
        n = 0..10
[scheduling]
    [[dependencies]]
        graph = """
prep => init<r> => sim<r,n> => post<r,n> => comb<r> => stop
    post<r=2,n=1> => boo  # a special case"""
[runtime]
    [[root]]
        script = """
r=${CYLC_TASK_PARAM_r:-}
n=${CYLC_TASK_PARAM_n:-}
[[ -n $r ]] && echo "parameter r is $r"
[[ -n $n ]] && echo "parameter n is $n"
sleep 5"""
    [[ENDS]]
    [[prep, stop]]
        inherit = None, ENDS
    [[RUN<r>]]
    [[init<r>, comb<r>, sim<r,n>, post<r,n>]]
        inherit = RUN<r>
[visualization]
    use node color for edges = True
    [[node attributes]]
        RUN<r=0> = "color=red"
        RUN<r=1> = "color=blue"
        RUN<r=2> = "color=darkgreen"
        comb<r> = "color=orange"
        ENDS = "style=filled", "fillcolor=gray"
        boo = "style=filled", "fillcolor=red"
