[cylc]
   [[reference test]]
       required run mode = live
       live mode suite timeout = 1.0 # minutes

[scheduling]
    [[dependencies]]
        graph = """a | b => c & d
                   c | d => e"""
[runtime]
    [[b,c]]
        script = sleep 10
    [[a,d, e]]
        script = true
