
[cylc]
    [[reference test]]
        required run mode = live
        live mode suite timeout = 0.5

[scheduling]
    initial cycle time = 2012010100
    final cycle time = 2012010118
        [[special tasks]]
            sequential = b
    [[dependencies]]
        graph = "a"
        [[[0,6,12,18]]]
            graph = "a => b"
[runtime]
    [[a,b]]
        script = "/bin/true" # fast
